#Hbonding dynamics

import numpy as py
from numpy import sqrt, arccos, average, std, gradient
from statistics import pstdev
import csv
import math
import matplotlib.pyplot as plt
from scipy.constants import e, epsilon_0, pi, N_A

#Cutoff distances

distH2OH2O=3.31
distOwHw=2.45
distClH2O=3.81
distKH2O=3.5
Excluded_Cl=3.15
Excluded_K=2.65

#Box dimension
Box_dimension=27.23
distEdge=Box_dimension-distH2OH2O

#Formatting .ato file

def ImportFile(filename):
    try:
        filename = str(filename)
        with open(filename, 'r') as f:
            reader = csv.reader(f, delimiter =' ',quotechar =',',quoting=csv.QUOTE_MINIMAL)
            Raw_Data = list(reader)
        return Raw_Data
    except:
        print("Problem with importing file")

def RemoveEmptySpaces(your_list):
    Currated_dataset = []
    for i in range(0,len(your_list)):
        list_gaps_removed = []
        for j in your_list[i]:
            try:
                Num = float(j)
                list_gaps_removed.append(Num)
            except:
                continue
        Currated_dataset.append(list_gaps_removed)
    return Currated_dataset


filename = "KCl24M020720box.ato"

def MoleculeLinePosGenerator(initial_pos,No_of_molecules,Line_spacing):
    Molecule_line_positions = []
    for i in range(initial_pos,(No_of_molecules*Line_spacing),Line_spacing):
        Molecule_line_positions.append(int(i))
    return Molecule_line_positions
    
    
def PullMoleculeData(Formatted_dataset,initial_pos,No_of_molecules,Line_spacing):
    #Write this function to pull specific rows of data corresponding to specific molecule in the simulation
    Pulled_List = []
    Molecule_line_positions = MoleculeLinePosGenerator(initial_pos,No_of_molecules,Line_spacing)
    for i in Molecule_line_positions:
        Pulled_List.append(Formatted_dataset[i])
    return Pulled_List
    
def PullAtomData(Formatted_dataset,initial_pos,No_of_molecules,Line_spacing):
    #Write this function to pull specific rows of data corresponding to specific molecule in the simulation
    Pulled_List = []
    Molecule_line_positions = MoleculeLinePosGenerator(initial_pos,No_of_molecules,Line_spacing)
    for i in Molecule_line_positions:
        Pulled_List.append(Formatted_dataset[i])
    return Pulled_List

def FormatDatasetH2O():
#This series of functions formats the ato file to pull out the H2O coordinates
    Raw_Data = ImportFile(filename)
    Formatted_dataset = RemoveEmptySpaces(Raw_Data)
    H2O_list = PullMoleculeData(Formatted_dataset,2,5000,11)
    return H2O_list   
    
def FormatDatasetOw():
#This series of functions formats the ato file to pull out the Ow coordinates
    Raw_Data = ImportFile(filename)
    Formatted_dataset = RemoveEmptySpaces(Raw_Data)
    Ow_list = PullAtomData(Formatted_dataset,4,5000,11)
    return Ow_list
    
def FormatDatasetHw1():
#This series of functions formats the ato file to pull out the Hw1 coordinates
    Raw_Data = ImportFile(filename)
    Formatted_dataset = RemoveEmptySpaces(Raw_Data)
    Hw1_list = PullAtomData(Formatted_dataset,7,5000,11)
    return Hw1_list
    
def FormatDatasetHw2():
#This series of functions formats the ato file to pull out the Hw2 coordinates
    Raw_Data = ImportFile(filename)
    Formatted_dataset = RemoveEmptySpaces(Raw_Data)
    Hw2_list = PullAtomData(Formatted_dataset,10,5000,11)
    return Hw2_list

def FormatDatasetCl():
#This series of functions formats the ato file to pull out the Br coordinates. No need to account for COM addition, single ion located at COM
    Raw_Data = ImportFile(filename)
    Formatted_dataset = RemoveEmptySpaces(Raw_Data)
    Cl_list = PullMoleculeData(Formatted_dataset,55002,11240,5)
    return Cl_list

def FormatDatasetK():
#This series of functions formats the ato file to pull out the K coordinates
    Raw_Data = ImportFile(filename)
    Formatted_dataset = RemoveEmptySpaces(Raw_Data)
    K_list = PullMoleculeData(Formatted_dataset,56202,11480,5)
    return K_list

#The next lines of code create lists of the coordinates of the atoms relative to the centre of the appropriate molecules

H2Ocoordfin=[]
Owcoord=[]
Hw1coord=[]
Hw2coord=[]
Clcoordfin=[]
Kcoordfin=[]

H2O_list=FormatDatasetH2O()
for i in range(0,len(H2O_list)):
    H2Ocoordfin.append(H2O_list[i][1:4])
    
Ow_list=FormatDatasetOw()
for i in range(0,len(Ow_list)):
    Owcoord.append(Ow_list[i][0:3])
    
Hw1_list=FormatDatasetHw1()
for i in range(0,len(Hw1_list)):
    Hw1coord.append(Hw1_list[i][0:3])
    
Hw2_list=FormatDatasetHw2()
for i in range(0,len(Hw2_list)):
    Hw2coord.append(Hw2_list[i][0:3])
    
Cl_list=FormatDatasetCl()
for i in range(0,len(Cl_list)):
    Clcoordfin.append(Cl_list[i][1:4])
    
K_list=FormatDatasetK()
for i in range(0,len(K_list)):
    Kcoordfin.append(K_list[i][1:4])
    
#The next lines of code create lists of the coordinates of the atoms relative to the centre of the box

Owcoordfin=[]
Hw1coordfin=[]
Hw2coordfin=[]
    
for i in range(0,len(Owcoord)):
    Owcoordfin.append(py.add(H2Ocoordfin[i],Owcoord[i]))
    
for i in range(0,len(Hw1coord)):
    Hw1coordfin.append(py.add(H2Ocoordfin[i],Hw1coord[i]))
    
for i in range(0,len(Hw2coord)):
    Hw2coordfin.append(py.add(H2Ocoordfin[i],Hw2coord[i]))
    
#Adds molecule numbers to the lists of molecular coordinates from 0 to max

for i in range(0,len(H2Ocoordfin)):
    H2Ocoordfin[i].append(i)

for i in range(0,len(Clcoordfin)):
    Clcoordfin[i].append(i)
    
for i in range(0,len(Kcoordfin)):
    Kcoordfin[i].append(i)
    
#Creates list of molecule numbers that are at the edge of the box

edge=[]
for i in range(0,len(H2Ocoordfin)):
    for j in H2Ocoordfin[i][0:3]:
        if abs(j)>(distEdge):
            edge.append(H2Ocoordfin[i][-1])
        else:
            continue
        
#Next lines of code identifies the molecules within a given Br ion hydration shell
    
Clhydrationshellmolecules=[]
    
def Clhydrationshell(molecule_no):
    Clhydrationshellzmolecules=[]
    for i in range(0,len(H2Ocoordfin)):
        if sqrt(((Clcoordfin[molecule_no][0]-H2Ocoordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-H2Ocoordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-H2Ocoordfin[i][2])**2))<distClH2O:
            Clhydrationshellzmolecules.append(H2Ocoordfin[i])
        else:
            continue
    Clhydrationshellmolecules.append(Clhydrationshellzmolecules)
    return Clhydrationshellmolecules

for j in range(0,len(Clcoordfin)):
    Clhydrationshellcoord=Clhydrationshell(j)
    
#Next lines of code identifies the molecules within a given K ion hydration shell

Khydrationshellmolecules=[]
    
def Khydrationshell(molecule_no):
    Khydrationshellzmolecules=[]
    for i in range(0,len(H2Ocoordfin)):
        if sqrt(((Kcoordfin[molecule_no][0]-H2Ocoordfin[i][0])**2)+((Kcoordfin[molecule_no][1]-H2Ocoordfin[i][1])**2)+((Kcoordfin[molecule_no][2]-H2Ocoordfin[i][2])**2))<distKH2O:
            Khydrationshellzmolecules.append(H2Ocoordfin[i])
        else:
            continue
    Khydrationshellmolecules.append(Khydrationshellzmolecules)
    return Khydrationshellmolecules

for j in range(0,len(Kcoordfin)):
    Perchloratehydrationshellcoord=Khydrationshell(j)
    
#Pulls out molecular numbers for Mg hydration shells

Clhydrationshellmolecule_nos=[]

def Clhydrationshell_nos(molecule_no):
    Clhydrationshellmoleculez_nos=[]
    for i in range(0,len(Clhydrationshellmolecules[molecule_no])):
        Clhydrationshellmoleculez_nos.append(Clhydrationshellmolecules[molecule_no][i][-1])
    Clhydrationshellmolecule_nos.append(Clhydrationshellmoleculez_nos)
    return Clhydrationshellmolecule_nos

for i in range(0,len(Clcoordfin)):
    Clhydrationshellmoleculenos=Clhydrationshell_nos(i)
    
#Unbroken list of water molecules hydrating Mg

MASTERClhydration_nos=sum(Clhydrationshellmolecule_nos,[])
    
#Pulls out molecular numbers for Mg hydration shells

Khydrationshellmolecule_nos=[]

def Khydrationshell_nos(molecule_no):
    Khydrationshellmoleculez_nos=[]
    for i in range(0,len(Khydrationshellmolecules[molecule_no])):
        Khydrationshellmoleculez_nos.append(Khydrationshellmolecules[molecule_no][i][-1])
    Khydrationshellmolecule_nos.append(Khydrationshellmoleculez_nos)
    return Khydrationshellmolecule_nos

for i in range(0,len(Kcoordfin)):
    Khydrationshellmoleculenos=Khydrationshell_nos(i)
    
#Unbroken list of water molecules hydrating perchlorate

MASTERKhydration_nos=sum(Khydrationshellmolecule_nos,[])

#Unbroken list of water molecules hydrating solutes with no duplicates in order from lowest to highest

MASTERHYDRATION_nos=list(set(MASTERClhydration_nos+MASTERKhydration_nos))

#Unbroken list of water molecules at edge of box with no duplicates in order from lowest to highest
        
edge_nos=list(set(edge))

#List of molecule numbers for comparison

nums=[]
for i in range(0,len(H2Ocoordfin)):
    nums.append(i)
    
#Unbroken list of all molecules solvating a solute or at the edge of the box in order from lowest to highest with no duplicates
    
BAD_MOLECULE_NOS=list(set(MASTERHYDRATION_nos+edge_nos))
    
mid_nos=[x for x in nums if x not in BAD_MOLECULE_NOS]

#Determines excluded volume and excluded volume fraction for box

Excluded_vol_per_Cl = (4/3)*pi*(Excluded_Cl**3)
Excluded_vol_Cl = Excluded_vol_per_Cl*len(Clcoordfin)
Excluded_vol_per_K = (4/3)*pi*(Excluded_K**3)
Excluded_vol_K = Excluded_vol_per_K*len(Kcoordfin)
Box_vol = (Box_dimension*2)**3
Excluded_frac=(Excluded_vol_Cl+Excluded_vol_K)/Box_vol
Bulk_frac=1.0-Excluded_frac

#Next lines of code identifies the molecules within a given water molecules spherical hydration shell

Hydrationshellmolecules=[]

def Hydrationshell(molecule_no):
    Hydrationshellzmolecules=[]
    for i in range(0,len(H2Ocoordfin)):
        if 0.5<sqrt(((Owcoordfin[molecule_no][0]-Owcoordfin[i][0])**2)+((Owcoordfin[molecule_no][1]-Owcoordfin[i][1])**2)+((Owcoordfin[molecule_no][2]-Owcoordfin[i][2])**2))<=distH2OH2O:
            Hydrationshellzmolecules.append(H2Ocoordfin[i])
        else:
            continue
    Hydrationshellmolecules.append(Hydrationshellzmolecules)
    return Hydrationshellmolecules

for j in range(0,len(H2Ocoordfin)):
    Hydrationshellcoord=Hydrationshell(j)

#Pulls out hydration numbers for a central water molecule
    
midHydration_nos=[]
    
for i in mid_nos:
    midHydration_nosz=[]
    for j in range(0,len(Hydrationshellmolecules[i])):
        midHydration_nosz.append(Hydrationshellmolecules[i][j][-1])
    midHydration_nos.append(midHydration_nosz)

#Determining which hydrogens are hydrogen bond donors to central water molecule (lie within first peak in RDF)

Hydrogen1bonddonors=[]
Hydrogen2bonddonors=[]

def H1bonddonor(molecule_no):
    Hydrogen1bonddonorsz=[]
    for i in midHydration_nos[molecule_no]:
        if 1.1<sqrt(((Owcoordfin[mid_nos[molecule_no]][0]-Hw1coordfin[i][0])**2)+((Owcoordfin[mid_nos[molecule_no]][1]-Hw1coordfin[i][1])**2)+((Owcoordfin[mid_nos[molecule_no]][2]-Hw1coordfin[i][2])**2))<=distOwHw:
            Hydrogen1bonddonorsz.append(i)
        else:
            continue
    Hydrogen1bonddonors.append(Hydrogen1bonddonorsz)
    return Hydrogen1bonddonors
    
for i in range(len(mid_nos)):
    H1donorbonds=H1bonddonor(i)
    
def H2bonddonor(molecule_no):
    Hydrogen2bonddonorsz=[]
    for i in midHydration_nos[molecule_no]:
        if 1.1<sqrt(((Owcoordfin[mid_nos[molecule_no]][0]-Hw2coordfin[i][0])**2)+((Owcoordfin[mid_nos[molecule_no]][1]-Hw2coordfin[i][1])**2)+((Owcoordfin[mid_nos[molecule_no]][2]-Hw2coordfin[i][2])**2))<=distOwHw:
            Hydrogen2bonddonorsz.append(i)
        else:
            continue
    Hydrogen2bonddonors.append(Hydrogen2bonddonorsz)
    return Hydrogen2bonddonors
        
for i in range(len(mid_nos)):
    H2donorbonds=H2bonddonor(i)
    
#Single list of hydrogen bond donors to get length

Hydrogenbonddonors=Hydrogen1bonddonors+Hydrogen2bonddonors
Flatdonorlist=[]
for sublist in Hydrogenbonddonors:
    for i in sublist:
        Flatdonorlist.append(i)
        
#Combines Hydrogen1bonddonors and Hydrogen2bonddonors for each central molecule

HBonddonors_merged=[]
for i in range(0,len(mid_nos)):
    HBonddonors_merged.append(Hydrogen1bonddonors[i]+Hydrogen2bonddonors[i])
    
#List of coordination numbers

coord_nos=[]
for i in mid_nos:
    coord_nos.append(len(Hydrationshellcoord[i]))
    
#calculates average coordination number
    
coord_no=sum(coord_nos)/len(coord_nos)

#Groups water molecules by coordination number

midcoord1=[]
midcoord2=[]
midcoord3=[]
midcoord4=[]
midcoord5=[]
midcoord6=[]
midcoord7=[]
midcoord8=[]

for i in range(0,len(mid_nos)):
    if len(Hydrationshellcoord[mid_nos[i]])==1:
        midcoord1.append(i)
    if len(Hydrationshellcoord[mid_nos[i]])==2:
        midcoord2.append(i)
    if len(Hydrationshellcoord[mid_nos[i]])==3:
        midcoord3.append(i)
    if len(Hydrationshellcoord[mid_nos[i]])==4:
        midcoord4.append(i)
    if len(Hydrationshellcoord[mid_nos[i]])==5:
        midcoord5.append(i)
    if len(Hydrationshellcoord[mid_nos[i]])==6:
        midcoord6.append(i)
    if len(Hydrationshellcoord[mid_nos[i]])==7:
        midcoord7.append(i)
    if len(Hydrationshellcoord[mid_nos[i]])==8:
        midcoord8.append(i)
        
#Calculating total interaction energy of water molecule within its hydration shell

midHydrationshellEnergies=[]

EpsilonOw=0.650
SigmaOw=3.166
ChargeOw=-0.84760

EpsilonHw=0
SigmaHw=0
ChargeHw=0.42380

K=1/(4*pi*epsilon_0)

def WaterBondEnergy1(molecule_no):
    midHydrationshellEnergies1z=[]
    for i in midHydration_nos[molecule_no]:
#Coulomb potential
        EqOw0Ow1=(K*ChargeOw*ChargeOw*(e**2))/(sqrt(((Owcoordfin[mid_nos[molecule_no]][0]-Owcoordfin[i][0])**2)+((Owcoordfin[mid_nos[molecule_no]][1]-Owcoordfin[i][1])**2)+((Owcoordfin[mid_nos[molecule_no]][2]-Owcoordfin[i][2])**2))*10**-10)
        EqOw0Hw11=(K*ChargeOw*ChargeHw*(e**2))/(sqrt(((Owcoordfin[mid_nos[molecule_no]][0]-Hw1coordfin[i][0])**2)+((Owcoordfin[mid_nos[molecule_no]][1]-Hw1coordfin[i][1])**2)+((Owcoordfin[mid_nos[molecule_no]][2]-Hw1coordfin[i][2])**2))*10**-10)
        EqOw0Hw21=(K*ChargeOw*ChargeHw*(e**2))/(sqrt(((Owcoordfin[mid_nos[molecule_no]][0]-Hw2coordfin[i][0])**2)+((Owcoordfin[mid_nos[molecule_no]][1]-Hw2coordfin[i][1])**2)+((Owcoordfin[mid_nos[molecule_no]][2]-Hw2coordfin[i][2])**2))*10**-10)
        EqHw10Ow1=(K*ChargeHw*ChargeOw*(e**2))/(sqrt(((Hw1coordfin[mid_nos[molecule_no]][0]-Owcoordfin[i][0])**2)+((Hw1coordfin[mid_nos[molecule_no]][1]-Owcoordfin[i][1])**2)+((Hw1coordfin[mid_nos[molecule_no]][2]-Owcoordfin[i][2])**2))*10**-10)
        EqHw10Hw11=(K*ChargeHw*ChargeHw*(e**2))/(sqrt(((Hw1coordfin[mid_nos[molecule_no]][0]-Hw1coordfin[i][0])**2)+((Hw1coordfin[mid_nos[molecule_no]][1]-Hw1coordfin[i][1])**2)+((Hw1coordfin[mid_nos[molecule_no]][2]-Hw1coordfin[i][2])**2))*10**-10)
        EqHw10Hw21=(K*ChargeHw*ChargeHw*(e**2))/(sqrt(((Hw1coordfin[mid_nos[molecule_no]][0]-Hw2coordfin[i][0])**2)+((Hw1coordfin[mid_nos[molecule_no]][1]-Hw2coordfin[i][1])**2)+((Hw1coordfin[mid_nos[molecule_no]][2]-Hw2coordfin[i][2])**2))*10**-10)
        EqHw20Ow1=(K*ChargeHw*ChargeOw*(e**2))/(sqrt(((Hw2coordfin[mid_nos[molecule_no]][0]-Owcoordfin[i][0])**2)+((Hw2coordfin[mid_nos[molecule_no]][1]-Owcoordfin[i][1])**2)+((Hw2coordfin[mid_nos[molecule_no]][2]-Owcoordfin[i][2])**2))*10**-10)
        EqHw20Hw11=(K*ChargeHw*ChargeHw*(e**2))/(sqrt(((Hw2coordfin[mid_nos[molecule_no]][0]-Hw1coordfin[i][0])**2)+((Hw2coordfin[mid_nos[molecule_no]][1]-Hw1coordfin[i][1])**2)+((Hw2coordfin[mid_nos[molecule_no]][2]-Hw1coordfin[i][2])**2))*10**-10)
        EqHw20Hw21=(K*ChargeHw*ChargeHw*(e**2))/(sqrt(((Hw2coordfin[mid_nos[molecule_no]][0]-Hw2coordfin[i][0])**2)+((Hw2coordfin[mid_nos[molecule_no]][1]-Hw2coordfin[i][1])**2)+((Hw2coordfin[mid_nos[molecule_no]][2]-Hw2coordfin[i][2])**2))*10**-10)
#Gives answer for Coulomb potential in joules    
        EqTotal=EqOw0Ow1+EqOw0Hw11+EqOw0Hw21+EqHw10Ow1+EqHw10Hw11+EqHw10Hw21+EqHw20Ow1+EqHw20Hw11+EqHw20Hw21
#Gives answer in for Coulomb potential kJ/mol
        EqTotalkJmol=EqTotal*(N_A)/1000
#VdW potential in kJ/mol
        EvdwOw0Ow1=(4*sqrt(EpsilonOw*EpsilonOw))*(((((SigmaOw+SigmaOw)/2)/sqrt(((Owcoordfin[mid_nos[molecule_no]][0]-Owcoordfin[i][0])**2)+((Owcoordfin[mid_nos[molecule_no]][1]-Owcoordfin[i][1])**2)+((Owcoordfin[mid_nos[molecule_no]][2]-Owcoordfin[i][2])**2)))**12)-((((SigmaOw+SigmaOw)/2)/sqrt(((Owcoordfin[mid_nos[molecule_no]][0]-Owcoordfin[i][0])**2)+((Owcoordfin[mid_nos[molecule_no]][1]-Owcoordfin[i][1])**2)+((Owcoordfin[mid_nos[molecule_no]][2]-Owcoordfin[i][2])**2)))**6))
#No need to consider hydrogens in water-water VdW interaction as both epsilon and sigma values are set to 0
        ETotal=EqTotalkJmol+EvdwOw0Ow1
        midHydrationshellEnergies1z.append(ETotal)
    midHydrationshellEnergies.append(midHydrationshellEnergies1z)
    return midHydrationshellEnergies

for i in range(len(mid_nos)):
    midHydrationshellEnergies1=WaterBondEnergy1(i)
    
midHydrationshellEnergiesTOTAL=[]

#Sum of interaction energies for central molecule within its hydration shell

for i in range(len(mid_nos)):
    midHydrationshellEnergiesTOTAL.append(sum(midHydrationshellEnergies[i]))

#Calculating total interaction energy for K cation with surrounding water molecules
    
KHydrationEnergies=[]

EpsilonK=0.51440
SigmaK=2.9400
ChargeK=1.0

def KBondEnergy(molecule_no):
    KHydrationEnergiesz=[]
    for i in Khydrationshellmolecule_nos[molecule_no]:
#Coulomb potential
        EqKOw=(K*ChargeK*ChargeOw*(e**2))/(sqrt(((Kcoordfin[molecule_no][0]-Owcoordfin[i][0])**2)+((Kcoordfin[molecule_no][1]-Owcoordfin[i][1])**2)+((Kcoordfin[molecule_no][2]-Owcoordfin[i][2])**2))*10**-10)
        EqKHw1=(K*ChargeK*ChargeHw*(e**2))/(sqrt(((Kcoordfin[molecule_no][0]-Hw1coordfin[i][0])**2)+((Kcoordfin[molecule_no][1]-Hw1coordfin[i][1])**2)+((Kcoordfin[molecule_no][2]-Hw1coordfin[i][2])**2))*10**-10)
        EqKHw2=(K*ChargeK*ChargeHw*(e**2))/(sqrt(((Kcoordfin[molecule_no][0]-Hw2coordfin[i][0])**2)+((Kcoordfin[molecule_no][1]-Hw2coordfin[i][1])**2)+((Kcoordfin[molecule_no][2]-Hw2coordfin[i][2])**2))*10**-10)
#Gives answer for Coulomb potential in joules    
        EqTotal=EqKOw+EqKHw1+EqKHw2
#Gives answer in for Coulomb potential kJ/mol
        EqTotalkJmol=EqTotal*(N_A)/1000
#VdW potential in kJ/mol
        EvdwKOw=(4*sqrt(EpsilonK*EpsilonOw))*(((((SigmaK+SigmaOw)/2)/sqrt(((Kcoordfin[molecule_no][0]-Owcoordfin[i][0])**2)+((Kcoordfin[molecule_no][1]-Owcoordfin[i][1])**2)+((Kcoordfin[molecule_no][2]-Owcoordfin[i][2])**2)))**12)-((((SigmaK+SigmaOw)/2)/sqrt(((Kcoordfin[molecule_no][0]-Owcoordfin[i][0])**2)+((Kcoordfin[molecule_no][1]-Owcoordfin[i][1])**2)+((Kcoordfin[molecule_no][2]-Owcoordfin[i][2])**2)))**6))
#No need to consider hydrogens in water-water VdW interaction as both epsilon and sigma values are set to 0
        ETotal=EqTotalkJmol+EvdwKOw
        KHydrationEnergiesz.append(ETotal)
    KHydrationEnergies.append(KHydrationEnergiesz)
    return KHydrationEnergies

for i in range(0,len(Khydrationshellmolecule_nos)):
    KHydrationE=KBondEnergy(i)
    
KHydrationshellEnergiesTOTAL=[]

#Sum of interaction energies for central molecule within its hydration shell

for i in range(len(Khydrationshellmolecule_nos)):
    KHydrationshellEnergiesTOTAL.append(sum(KHydrationEnergies[i]))
    
#Calculating total interaction energy for anion with surrounding water molecules

ClHydrationEnergies=[]

EpsilonCl=0.56600
SigmaCl=4.1910
ChargeCl=-1.0

def ClBondEnergy(molecule_no):
    ClHydrationEnergiesz=[]
    for i in Clhydrationshellmolecule_nos[molecule_no]:
#Coulomb potential
        EqClOw=(K*ChargeCl*ChargeOw*(e**2))/(sqrt(((Clcoordfin[molecule_no][0]-Owcoordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-Owcoordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-Owcoordfin[i][2])**2))*10**-10)
        EqClHw1=(K*ChargeCl*ChargeHw*(e**2))/(sqrt(((Clcoordfin[molecule_no][0]-Hw1coordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-Hw1coordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-Hw1coordfin[i][2])**2))*10**-10)
        EqClHw2=(K*ChargeCl*ChargeHw*(e**2))/(sqrt(((Clcoordfin[molecule_no][0]-Hw2coordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-Hw2coordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-Hw2coordfin[i][2])**2))*10**-10)
#Gives answer for Coulomb potential in joules    
        EqTotal=EqClOw+EqClHw1+EqClHw2
#Gives answer in for Coulomb potential kJ/mol
        EqTotalkJmol=EqTotal*(N_A)/1000
#VdW potential in kJ/mol
        EvdwClOw=(4*sqrt(EpsilonCl*EpsilonOw))*(((((SigmaCl+SigmaOw)/2)/sqrt(((Clcoordfin[molecule_no][0]-Owcoordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-Owcoordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-Owcoordfin[i][2])**2)))**12)-((((SigmaCl+SigmaOw)/2)/sqrt(((Clcoordfin[molecule_no][0]-Owcoordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-Owcoordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-Owcoordfin[i][2])**2)))**6))
#No need to consider hydrogens in water-water VdW interaction as both epsilon and sigma values are set to 0
        ETotal=EqTotalkJmol+EvdwClOw
        ClHydrationEnergiesz.append(ETotal)
    ClHydrationEnergies.append(ClHydrationEnergiesz)
    return ClHydrationEnergies

for i in range(0,len(Clhydrationshellmolecule_nos)):
    ClHydrationE=ClBondEnergy(i)
    
ClHydrationshellEnergiesTOTAL=[]

#Sum of interaction energies for central molecule within its hydration shell

for i in range(len(Clhydrationshellmolecule_nos)):
    ClHydrationshellEnergiesTOTAL.append(sum(ClHydrationEnergies[i]))

#Hydrogen bond interaction energies
  
midHBondEnergies=[] 
    
def HBondE(molecule_no):
    midHBondEnergiesz=[]
    for i in HBonddonors_merged[molecule_no]:
        midHBondEnergiesz.append(midHydrationshellEnergies[molecule_no][midHydration_nos[molecule_no].index(i)])
    midHBondEnergies.append(midHBondEnergiesz)
    return midHBondEnergies

for i in range(len(mid_nos)):
    HBondEnergies=HBondE(i)
    
#Single array containing molecule number, coordination number, and total interaction energy

INFO=[]

def INFOgenerator(molecule_no):
    INFOz=[]
    INFOz.append(mid_nos[molecule_no])
    INFOz.append(len(Hydrationshellcoord[mid_nos[molecule_no]]))
    INFOz.append(midHydrationshellEnergiesTOTAL[molecule_no])
    INFO.append(INFOz)
    return INFO

for i in range(0,len(mid_nos)):
    INFO1=INFOgenerator(i)
    
#Total interaction energy by coordination number

coordE1=[]
coordE2=[]
coordE3=[]
coordE4=[]
coordE5=[]
coordE6=[]
coordE7=[]
coordE8=[]

for i in midcoord1:
    coordE1.append(midHydrationshellEnergiesTOTAL[i])
for i in midcoord2:
    coordE2.append(midHydrationshellEnergiesTOTAL[i])
for i in midcoord3:
    coordE3.append(midHydrationshellEnergiesTOTAL[i])
for i in midcoord4:
    coordE4.append(midHydrationshellEnergiesTOTAL[i])
for i in midcoord5:
    coordE5.append(midHydrationshellEnergiesTOTAL[i])
for i in midcoord6:
    coordE6.append(midHydrationshellEnergiesTOTAL[i])
for i in midcoord7:
    coordE7.append(midHydrationshellEnergiesTOTAL[i])
for i in midcoord8:
    coordE8.append(midHydrationshellEnergiesTOTAL[i])
    
coord_no_list=[1,2,3,4,5,6,7,8]
coordElist=[average(coordE1),average(coordE2),average(coordE3),average(coordE4),average(coordE5),average(coordE6),average(coordE7),average(coordE8)]
coordElistlen=[len(coordE1),len(coordE2),len(coordE3),len(coordE4),len(coordE5),len(coordE6),len(coordE7),len(coordE8)]
coordElistpermol=[average(coordE1)/1,average(coordE2)/2,average(coordE3)/3,average(coordE4)/4,average(coordE5)/5,average(coordE6)/6,average(coordE7)/7,average(coordE8)/8]
coordstderrlist=[std(coordE1)/len(coordE1),std(coordE2)/len(coordE2),std(coordE3)/len(coordE3),std(coordE4)/len(coordE4),std(coordE5)/len(coordE5),std(coordE6)/len(coordE6),std(coordE7)/len(coordE7),std(coordE8)/len(coordE8)]
coordderivative=gradient(coordElist)
coord_no_list_derivative=coord_no_list[1:-1]
coordderivativeforfit=coordderivative[1:-1]

#Defining dipole on each water molecule
    
HHcoord=[]
def HHCOM(molecule_no):
    HHcoordz=[]
    HHvector=py.subtract(Hw1coordfin[molecule_no],Hw2coordfin[molecule_no])
    HHCOMcoord=py.add(Hw2coordfin[molecule_no],HHvector/2)
    for element in HHCOMcoord:
        HHcoordz.append(element)
    HHcoord.append(HHcoordz)
    return HHcoord

for i in range(0,len(H2Ocoordfin)):
    HHcoordFIN=HHCOM(i) 
    
Dipole=[]
def Dipolevector(molecule_no):
    Dipolez=[]
    Dipolevec=py.subtract(HHcoord[molecule_no],Owcoordfin[molecule_no])
    for element in Dipolevec:
        Dipolez.append(element)
    Dipole.append(Dipolez)
    return Dipole

for i in range(0,len(H2Ocoordfin)):
    DipoleFIN=Dipolevector(i)
    
#Calculating water dipole angle (angle between Ow-Ow vector and hydrating molecule dipole)

Dipoleangles=[]

def Dipoleangle(molecule_no):
    for i in midHydration_nos[molecule_no]:
        try:
            cosinealphanum=((Owcoordfin[mid_nos[molecule_no]][0]-Owcoordfin[i][0])*(HHcoord[i][0]-Owcoordfin[i][0]))+((Owcoordfin[mid_nos[molecule_no]][1]-Owcoordfin[i][1])*(HHcoord[i][1]-Owcoordfin[i][1]))+((Owcoordfin[mid_nos[molecule_no]][2]-Owcoordfin[i][2])*(HHcoord[i][2]-Owcoordfin[i][2]))
            cosinealphaden=(sqrt(((Owcoordfin[mid_nos[molecule_no]][0]-Owcoordfin[i][0])**2)+((Owcoordfin[mid_nos[molecule_no]][1]-Owcoordfin[i][1])**2)+((Owcoordfin[mid_nos[molecule_no]][2]-Owcoordfin[i][2])**2)))*(sqrt(((HHcoord[i][0]-Owcoordfin[i][0])**2)+((HHcoord[i][1]-Owcoordfin[i][1])**2)+((HHcoord[i][2]-Owcoordfin[i][2])**2)))
            cosinealpha=cosinealphanum/cosinealphaden
            alpharad=arccos(cosinealpha)
            alphadeg=math.degrees(alpharad)
            Dipoleangles.append(alphadeg)
        except:
            continue
    return Dipoleangles

for i in range(len(mid_nos)):    
    Dipangles=Dipoleangle(i)
    
#Calculating Mg water dipole angle (Angle between Mg-Ow vector and hydrating molecule dipole)

ClDipoleangles=[]

def ClDipoleangle(molecule_no):
    for i in Clhydrationshellmolecule_nos[molecule_no]:
        try:
            cosinealphanum=((Clcoordfin[molecule_no][0]-Owcoordfin[i][0])*(HHcoord[i][0]-Owcoordfin[i][0]))+((Clcoordfin[molecule_no][1]-Owcoordfin[i][1])*(HHcoord[i][1]-Owcoordfin[i][1]))+((Clcoordfin[molecule_no][2]-Owcoordfin[i][2])*(HHcoord[i][2]-Owcoordfin[i][2]))
            cosinealphaden=(sqrt(((Clcoordfin[molecule_no][0]-Owcoordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-Owcoordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-Owcoordfin[i][2])**2)))*(sqrt(((HHcoord[i][0]-Owcoordfin[i][0])**2)+((HHcoord[i][1]-Owcoordfin[i][1])**2)+((HHcoord[i][2]-Owcoordfin[i][2])**2)))
            cosinealpha=cosinealphanum/cosinealphaden
            alpharad=arccos(cosinealpha)
            alphadeg=math.degrees(alpharad)
            ClDipoleangles.append(alphadeg)
        except:
            continue
    return ClDipoleangles

for i in range(len(Clcoordfin)):    
    ClDipangles=ClDipoleangle(i)

#Calculating Clp water dipole angle (Angle between Clp-Ow vector and hydrating molecule dipole)

KDipoleangles=[]

def KDipoleangle(molecule_no):
    for i in Khydrationshellmolecule_nos[molecule_no]:
        try:
            cosinealphanum=((Kcoordfin[molecule_no][0]-Owcoordfin[i][0])*(HHcoord[i][0]-Owcoordfin[i][0]))+((Kcoordfin[molecule_no][1]-Owcoordfin[i][1])*(HHcoord[i][1]-Owcoordfin[i][1]))+((Kcoordfin[molecule_no][2]-Owcoordfin[i][2])*(HHcoord[i][2]-Owcoordfin[i][2]))
            cosinealphaden=(sqrt(((Kcoordfin[molecule_no][0]-Owcoordfin[i][0])**2)+((Kcoordfin[molecule_no][1]-Owcoordfin[i][1])**2)+((Kcoordfin[molecule_no][2]-Owcoordfin[i][2])**2)))*(sqrt(((HHcoord[i][0]-Owcoordfin[i][0])**2)+((HHcoord[i][1]-Owcoordfin[i][1])**2)+((HHcoord[i][2]-Owcoordfin[i][2])**2)))
            cosinealpha=cosinealphanum/cosinealphaden
            alpharad=arccos(cosinealpha)
            alphadeg=math.degrees(alpharad)
            KDipoleangles.append(alphadeg)
        except:
            continue
    return KDipoleangles

for i in range(len(Kcoordfin)):    
    KDipangles=KDipoleangle(i)

#Determining positive interaction energy fraction

FlatElist=[]

for sublist in midHydrationshellEnergies:
    for i in sublist:
        FlatElist.append(i)

PositiveE=[]
NegativeE=[]

for i in FlatElist:
    if i >= 0.0:
        PositiveE.append(i)
    else:
        NegativeE.append(i)
        
PosFrac=len(PositiveE)/(len(PositiveE)+len(NegativeE))

#Flattened HBond energy list

FlatHBondElist=[]
for sublist in midHBondEnergies:
    for i in sublist:
        FlatHBondElist.append(i)

#Flattened K hydration energy list per water molecule

FlatKlist=[]
for sublist in KHydrationEnergies:
    for i in sublist:
        FlatKlist.append(i)

#Flattened anion hydration energy list per water molecule

FlatCllist=[]
for sublist in ClHydrationEnergies:
    for i in sublist:
        FlatCllist.append(i)

#Determining cyclic dimers

Cyclicdimers11=[]

def Cyclicdimer11(molecule_no):
    for i in Hydrogen1bonddonors[molecule_no]:
        try:
            for j in Hydrogen1bonddonors[mid_nos.index(i)]:
                if j==mid_nos[molecule_no]:
                    Cyclicdimers11.append(i)
                else:
                    continue
        except:
            continue
    return Cyclicdimers11
    
for i in range(0,len(mid_nos)):
    CyclicDimers11=Cyclicdimer11(i)
    
Cyclicdimers12=[]

def Cyclicdimer12(molecule_no):
    for i in Hydrogen1bonddonors[molecule_no]:
        try:
            for j in Hydrogen2bonddonors[mid_nos.index(i)]:
                if j==mid_nos[molecule_no]:
                    Cyclicdimers12.append(i)
                else:
                    continue
        except:
            continue
    return Cyclicdimers12
    
for i in range(0,len(mid_nos)):
    CyclicDimers12=Cyclicdimer12(i)
    
Cyclicdimers21=[]

def Cyclicdimer21(molecule_no):
    for i in Hydrogen2bonddonors[molecule_no]:
        try:
            for j in Hydrogen1bonddonors[mid_nos.index(i)]:
                if j==mid_nos[molecule_no]:
                    Cyclicdimers21.append(i)
                else:
                    continue
        except:
            continue
    return Cyclicdimers21
    
for i in range(0,len(mid_nos)):
    CyclicDimers21=Cyclicdimer21(i)
    
Cyclicdimers22=[]

def Cyclicdimer22(molecule_no):
    for i in Hydrogen2bonddonors[molecule_no]:
        try:
            for j in Hydrogen2bonddonors[mid_nos.index(i)]:
                if j==mid_nos[molecule_no]:
                    Cyclicdimers22.append(i)
                else:
                    continue
        except:
            continue
    return Cyclicdimers22
    
for i in range(0,len(mid_nos)):
    CyclicDimers22=Cyclicdimer22(i)
    
CyclicDimers=Cyclicdimers11+Cyclicdimers12+Cyclicdimers21+Cyclicdimers22

CyclicDimerPairNo=(len(CyclicDimers)/2)
CyclicDimerPairFrac=(len(CyclicDimers)/len(mid_nos))

#Determining bifurcated oxygens

BifurcO=[]

def Bifurcatedoxygen(molecule_no):
    for i in Hydrogen1bonddonors[molecule_no]:
        try:
            for j in Hydrogen2bonddonors[molecule_no]:
                if i==j:
                    BifurcO.append(molecule_no)
                else:
                    continue
        except:
            continue
    return BifurcO
        
for i in range(0,len(mid_nos)):
    BifurcatedO=Bifurcatedoxygen(i)        
    
#Calculating bifurcated hydrogen bonds (one hydrogen donating a hydrogen bond to two different water molecules)

#Defining maximum shell for water molecules that can share a hydrogen bond

BifurcHmolecules=[]

def Bifurcshell(molecule_no):
    BifurcHzmolecules=[]
    for i in range(0,len(mid_nos)):
        if 0.5<sqrt(((H2Ocoordfin[mid_nos[molecule_no]][0]-H2Ocoordfin[mid_nos[i]][0])**2)+((H2Ocoordfin[mid_nos[molecule_no]][1]-H2Ocoordfin[mid_nos[i]][1])**2)+((H2Ocoordfin[mid_nos[molecule_no]][2]-H2Ocoordfin[mid_nos[i]][2])**2))<(2*distOwHw):
            BifurcHzmolecules.append(i)
        else:
            continue
    BifurcHmolecules.append(BifurcHzmolecules)
    return BifurcHmolecules
        
for i in range(0,len(mid_nos)):
    Bifurcshellnos=Bifurcshell(i)
    
#Adding hydrogen bond donors to all molecules in potential hydrogen bond sharing distance

BifurcHbonds1=[]

def Bifurcshellbonds1(molecule_no):   
    BifurcHzbonds1=[]
    for i in BifurcHmolecules[molecule_no]:
        BifurcHzbonds1.append(Hydrogen1bonddonors[i])
    BifurcHbonds1.append(BifurcHzbonds1)
    return BifurcHbonds1

for i in range(0,len(mid_nos)):
    BifurcshellHbonds1=Bifurcshellbonds1(i)
    
BifurcHbonds2=[]

def Bifurcshellbonds2(molecule_no):
    BifurcHzbonds2=[]
    for i in BifurcHmolecules[molecule_no]:
        BifurcHzbonds2.append(Hydrogen2bonddonors[i]) 
    BifurcHbonds2.append(BifurcHzbonds2)
    return BifurcHbonds2

for i in range(0,len(mid_nos)):
    BifurcshellHbonds2=Bifurcshellbonds2(i)
    
#Determining which molecules are sharing a hydrogen bond with central molecule

BifurcHbonds1nos=[]

def BifurcHnos1(molecule_no):
    for i in Hydrogen1bonddonors[molecule_no]:
        try:
            for j in BifurcHmolecules[molecule_no]:
                for k in Hydrogen1bonddonors[j]:
                    if k==i:
                        BifurcHbonds1nos.append(k)
                    else:
                        continue
        except:
            continue
    return BifurcHbonds1nos

for i in range(0,len(mid_nos)):
    BifurcHnumbers1=BifurcHnos1(i)

BifurcHbonds2nos=[]

def BifurcHnos2(molecule_no):
    for i in Hydrogen2bonddonors[molecule_no]:
        try:
            for j in BifurcHmolecules[molecule_no]:
                for k in Hydrogen2bonddonors[j]:
                    if k==i:
                        BifurcHbonds2nos.append(k)
                    else:
                        continue
        except:
            continue
    return BifurcHbonds2nos

for i in range(0,len(mid_nos)):
    BifurcHnumbers2=BifurcHnos2(i)
    
nobifurcHTOTAL=(len(BifurcHbonds1nos)+len(BifurcHbonds2nos))/2
fracbifurcHTOTAL=nobifurcHTOTAL/(len(Flatdonorlist))

#Plotting results

#Dipole angle distributions

plt.figure(1)
WWDipoleanglex=py.arange(0.0,188.0,2.0)
heights1, edges1 = py.histogram(Dipoleangles, bins=py.arange(0.0,190.0,2.0)-1.0)
plt.plot(WWDipoleanglex,heights1,"ro")
plt.xlabel("Dipole angle (degrees)")
plt.title("Water-water dipole angle distribution")

plt.figure(2)
ClWDipoleanglex=py.arange(0.0,188.0,2.0)
heights2,edges2 = py.histogram(ClDipoleangles, bins=py.arange(0.0,190.0,2.0)-1.0)
plt.plot(ClWDipoleanglex,heights2,"ro")
plt.xlabel("Dipole angle (degrees)")
plt.title("Cl-water dipole angle distribution")

plt.figure(3)
KWDipoleanglex=py.arange(0.0,188.0,2.0)
heights3, edges3 = py.histogram(KDipoleangles, bins=py.arange(0.0,190.0,2.0)-1.0)
plt.plot(KWDipoleanglex,heights3,"ro")
plt.xlabel("Dipole angle (degrees)")
plt.title("K-water dipole angle distribution")

plt.figure(4)
HBondEx=py.arange(-80.0,49.0,1.0)
heights4, edges4 = py.histogram(FlatHBondElist, bins=py.arange(-80.0,50.0,1.0)-0.5)
plt.plot(HBondEx,heights4,"ro")
plt.xlabel("Hydrogen bond interaction energy (kJ/mol)")
plt.title("Interaction energy between two hydrogen bonded water molecules")

plt.figure(5)
HydrationEx=py.arange(-80.0,49.0,1.0)
heights5, edges5 = py.histogram(FlatElist, bins=py.arange(-80.0,50.0,1.0)-0.5)
plt.plot(HydrationEx,heights5,"ro")
plt.xlabel("Interaction energy (kJ/mol)")
plt.title("Interaction energy between central water molecule and hydrating water molecule")

#Total interaction energy of central molecule vs coordination number
plt.figure(6)
plt.plot(coord_no_list,coordElist,"ro")
plt.xlabel("Coordination number")
plt.ylabel("Total interaction energy (kJ/mol)")
plt.errorbar(coord_no_list,coordElist,xerr=None,yerr=coordstderrlist)

#Total interaction energy of central molecule per coordinated molecule vs coordination number
plt.figure(7)
plt.plot(coord_no_list,coordElistpermol,"ro")
plt.xlabel("Coordination number")
plt.ylabel("Total interaction energy per coordinated molecule (kJ/mol)")

#Interaction energy per water molecule with central K ion

plt.figure(8)
KHydrationEx=py.arange(-120.0,39.0,1.0)
heights8, edges8 = py.histogram(FlatKlist, bins=py.arange(-120.0,40.0,1.0)-0.5)
plt.plot(KHydrationEx,heights8,"ro")
plt.xlabel("Interaction energy (kJ/mol)")
plt.title("Interaction energy between central K ion and hydrating water molecule (kJ/mol)")

plt.figure(9)
KHydrationETOTALx=py.arange(-500.0,-55.0,5.0)
heights9, edges9 = py.histogram(KHydrationshellEnergiesTOTAL, bins=py.arange(-500.0,-50.0,5.0)-2.5)
plt.plot(KHydrationETOTALx,heights9,"ro")
plt.xlabel("Interaction energy (kJ/mol)")
plt.title("Total K hydration interaction energy (kJ/mol)")

#Total interaction energy with central Cl ion

plt.figure(10)
ClHydrationEx=py.arange(-120.0,39.0,1.0)
heights10, edges10 = py.histogram(FlatCllist, bins=py.arange(-120.0,40.0,1.0)-0.5)
plt.plot(ClHydrationEx,heights10,"ro")
plt.xlabel("Interaction energy (kJ/mol)")
plt.title("Interaction energy between central Cl ion and hydrating water molecule (kJ/mol)")

plt.figure(11)
ClHydrationETOTALx=py.arange(-500.0,-55.0,5.0)
heights11, edges11 = py.histogram(ClHydrationshellEnergiesTOTAL, bins=py.arange(-500.0,-50.0,5.0)-2.5)
plt.plot(ClHydrationETOTALx,heights11,"ro")
plt.xlabel("Interaction energy (kJ/mol)")
plt.title("Total Cl hydration interaction energy (kJ/mol)")

plt.show()

#Arrays to list for output to csv

heights1list=py.array(heights1).tolist()
heights2list=py.array(heights2).tolist()
heights3list=py.array(heights3).tolist()
heights4list=py.array(heights4).tolist()
heights5list=py.array(heights5).tolist()
heights8list=py.array(heights8).tolist()
heights9list=py.array(heights9).tolist()
heights10list=py.array(heights10).tolist()
heights11list=py.array(heights11).tolist()

WWDipoleanglexlist=py.array(WWDipoleanglex).tolist()
ClWDipoleanglexlist=py.array(ClWDipoleanglex).tolist()
KWDipoleanglexlist=py.array(KWDipoleanglex).tolist()
HBondExlist=py.array(HBondEx).tolist()
HydrationExlist=py.array(HydrationEx).tolist()
KHydrationExlist=py.array(KHydrationEx).tolist()
KHydrationETOTALxlist=py.array(KHydrationETOTALx).tolist()
ClHydrationExlist=py.array(ClHydrationEx).tolist()
ClHydrationETOTALxlist=py.array(ClHydrationETOTALx).tolist()

#Writing results to CSV file KCl24M020720box.ato

file=open("KCl_2.4_M_HBOND_results_2.csv","a")
file.write("Results")
file.write("\n")
file.write("\n")
file.write("Average coordination number")
file.write(",")
file.write(str(coord_no))
file.write("\n")
file.write("\n")
file.write("Fraction of water molecules in first hydration shell with positive interaction energy : ")
file.write(",")
file.write(str(PosFrac))
file.write("\n")
file.write("\n")
file.write("Fraction of bulk water molecules in a cyclic dimer : ")
file.write(",")
file.write(str(CyclicDimerPairFrac))
file.write("\n")
file.write("\n")
file.write("NORMALISED Fraction of bulk water molecules in a cyclic dimer : ")
file.write(",")
file.write(str(CyclicDimerPairFrac/Bulk_frac))
file.write("\n")
file.write("\n")
file.write("Fraction of bulk water molecules with a bifurcated oxygen : ")
file.write(",")
file.write(str((len(BifurcatedO))/len(mid_nos)))
file.write("\n")
file.write("\n")
file.write("NORMALISED Fraction of bulk water molecules with a bifurcates oxygen : ")
file.write(",")
file.write(str(((len(BifurcatedO))/len(mid_nos))/Bulk_frac))
file.write("\n")
file.write("\n")
file.write("Fraction of bulk water molecules with a bifurcated hydrogen : ")
file.write(",")
file.write(str(fracbifurcHTOTAL))
file.write("\n")
file.write("\n")
file.write("NORMALISED Fraction of bulk water molecules with a bifurcated hydrogen : ")
file.write(",")
file.write(str(fracbifurcHTOTAL/Bulk_frac))
file.write("\n")
file.write("\n")
file.write("Water-Water Dipole Angle (degrees)")
file.write(",")
file.write(str(WWDipoleanglexlist).strip("[]"))
file.write("\n")
file.write("Occurance")
file.write(",")
file.write(str(heights1list).strip("[]"))
file.write("\n")
file.write("\n")
file.write("Cl-Water Dipole Angle (degrees)")
file.write(",")
file.write(str(ClWDipoleanglexlist).strip("[]"))
file.write("\n")
file.write("Occurance")
file.write(",")
file.write(str(heights2list).strip("[]"))
file.write("\n")
file.write("\n")
file.write("K-Water Dipole Angle (degrees)")
file.write(",")
file.write(str(KWDipoleanglexlist).strip("[]"))
file.write("\n")
file.write("Occurance")
file.write(",")
file.write(str(heights3list).strip("[]"))
file.write("\n")
file.write("\n")
file.write("Water-Water Hydrogen Bond Energy (kJ/mol)")
file.write(",")
file.write(str(HBondExlist).strip("[]"))
file.write("\n")
file.write("Occurance")
file.write(",")
file.write(str(heights4list).strip("[]"))
file.write("\n")
file.write("\n")
file.write("Hydration Water Interaction Energy")
file.write(",")
file.write(str(HydrationExlist).strip("[]"))
file.write("\n")
file.write("Occurance")
file.write(",")
file.write(str(heights5list).strip("[]"))
file.write("\n")
file.write("\n")
file.write("Coordination number : ")
file.write(",")
file.write(str(coord_no_list).strip("[]"))
file.write("\n")
file.write("Total interaction energy of central molecule : ")
file.write(",")
file.write(str(coordElist).strip("[]"))
file.write("\n")
file.write("\n")
file.write("K Hydration Water Interaction Energy")
file.write(",")
file.write(str(KHydrationExlist).strip("[]"))
file.write("\n")
file.write("Occurance")
file.write(",")
file.write(str(heights8list).strip("[]"))
file.write("\n")
file.write("\n")
file.write("TOTAL K Hydration Water Interaction Energy")
file.write(",")
file.write(str(KHydrationETOTALxlist).strip("[]"))
file.write("\n")
file.write("Occurance")
file.write(",")
file.write(str(heights9list).strip("[]"))
file.write("\n")
file.write("\n")
file.write("Cl Hydration Water Interaction Energy")
file.write(",")
file.write(str(ClHydrationExlist).strip("[]"))
file.write("\n")
file.write("Occurance")
file.write(",")
file.write(str(heights10list).strip("[]"))
file.write("\n")
file.write("\n")
file.write("TOTAL Cl Hydration Water Interaction Energy")
file.write(",")
file.write(str(ClHydrationETOTALxlist).strip("[]"))
file.write("\n")
file.write("Occurance")
file.write(",")
file.write(str(heights11list).strip("[]"))
file.write("\n")
file.write("\n")
file.close()

#Testing code

#Calcuating how ion-water interaction energy varies with distance

#Determining molecules within a given distance from central ion

distClH2OBIG=6.0
Clcutoffs=[]
for i in py.arange(1.0,distClH2OBIG,0.5):
    Clcutoffs.append(i)

Cl_EHydration_nos=[]
    
def Cl_Ehydrationshell(molecule_no):
    Cl_Ehydrationshellzmolecules=[]
    for i in range(0,len(H2Ocoordfin)):
        if sqrt(((Clcoordfin[molecule_no][0]-H2Ocoordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-H2Ocoordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-H2Ocoordfin[i][2])**2))<distClH2OBIG:
            Cl_Ehydrationshellzmolecules.append(H2Ocoordfin[i][-1])
        else:
            continue
    Cl_EHydration_nos.append(Cl_Ehydrationshellzmolecules)
    return Clhydrationshellmolecules

for j in range(0,len(Clcoordfin)):
    Cl_Ehydrationshellcoord=Cl_Ehydrationshell(j)
    
Clhydration_dist=[]

def Clhydration_distances(molecule_no):
    Clhydration_distz=[]
    for i in Cl_EHydration_nos[molecule_no]:
        Clhydration_distz.append(sqrt(((Clcoordfin[molecule_no][0]-H2Ocoordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-H2Ocoordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-H2Ocoordfin[i][2])**2)))
    Clhydration_dist.append(Clhydration_distz)
    return Clhydration_dist

for i in range(0,len(Clcoordfin)):
    Cldistances=Clhydration_distances(i)
    
#Calculating interaction energy
    
ClHydrationEnergies_H=[]

def ClBondEnergy_H(molecule_no):
    ClHydrationEnergies_Hz=[]
    for i in Cl_EHydration_nos[molecule_no]:
#Coulomb potential
        EqFOw=(K*ChargeCl*ChargeOw*(e**2))/(sqrt(((Clcoordfin[molecule_no][0]-Owcoordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-Owcoordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-Owcoordfin[i][2])**2))*10**-10)
        EqFHw1=(K*ChargeCl*ChargeHw*(e**2))/(sqrt(((Clcoordfin[molecule_no][0]-Hw1coordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-Hw1coordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-Hw1coordfin[i][2])**2))*10**-10)
        EqFHw2=(K*ChargeCl*ChargeHw*(e**2))/(sqrt(((Clcoordfin[molecule_no][0]-Hw2coordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-Hw2coordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-Hw2coordfin[i][2])**2))*10**-10)
#Gives answer for Coulomb potential in joules    
        EqTotal=EqFOw+EqFHw1+EqFHw2
#Gives answer in for Coulomb potential kJ/mol
        EqTotalkJmol=EqTotal*(N_A)/1000
#VdW potential in kJ/mol
        EvdwFOw=(4*sqrt(EpsilonCl*EpsilonOw))*(((((SigmaCl+SigmaOw)/2)/sqrt(((Clcoordfin[molecule_no][0]-Owcoordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-Owcoordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-Owcoordfin[i][2])**2)))**12)-((((SigmaCl+SigmaOw)/2)/sqrt(((Clcoordfin[molecule_no][0]-Owcoordfin[i][0])**2)+((Clcoordfin[molecule_no][1]-Owcoordfin[i][1])**2)+((Clcoordfin[molecule_no][2]-Owcoordfin[i][2])**2)))**6))
#No need to consider hydrogens in water-water VdW interaction as both epsilon and sigma values are set to 0
        ETotal=EqTotalkJmol+EvdwFOw
        ClHydrationEnergies_Hz.append(ETotal)
    ClHydrationEnergies_H.append(ClHydrationEnergies_Hz)
    return ClHydrationEnergies_H

for i in range(0,len(Cl_EHydration_nos)):
    ClHydrationE_H=ClBondEnergy_H(i)
    
#Grouping/averaging data

Cl_Elist=[]
Cl_distlist=[]
for sublist in ClHydrationEnergies_H:
    for i in sublist:
        Cl_Elist.append(i)

for sublist in Clhydration_dist:
    for i in sublist:
        Cl_distlist.append(i)

Cl_distlist, Cl_Elist = (list(t) for t in zip(*sorted(zip(Cl_distlist, Cl_Elist))))
        
plt.plot(Cl_distlist,Cl_Elist,"ro")
plt.show()

Cl_Dist=[]

def Cl_E_range(r):
    Cl_Distz=[]
    for element in Cl_distlist:
        if r<element<(r+0.25):
            Cl_Distz.append(element)
        else:
            continue
    Cl_Dist.append(Cl_Distz)
    return Cl_Dist

for i in py.arange(2.75,6.0,0.25):
    Cl_Distances=Cl_E_range(i)
    
Cl_Hlist=[]

for i in range(0,len(Cl_Dist)):
    Cl_Hlistz=[]
    for element in Cl_Dist[i]:
        Cl_Hlistz.append(Cl_Elist[Cl_distlist.index(element)])
    Cl_Hlist.append(Cl_Hlistz)
    
Cl_Hlistaverage=[]

for sublist in Cl_Hlist:
    x=(sum(sublist)/len(sublist))
    Cl_Hlistaverage.append(x)

Cl_Herr=[]
for sublist in Cl_Hlist:
    x=(pstdev(sublist)/(sqrt(len(sublist))))
    Cl_Herr.append(x)

Cl_Distaverage=[]    
for i in py.arange(2.75,6.0,0.25)+0.125:
    Cl_Distaverage.append(i)
    
plt.plot(Cl_Distaverage,Cl_Hlistaverage,"ro")
plt.show()

file=open("Enthalpy vs Distance.csv","a")
file.write("Distance")
file.write(",")
file.write(str(Cl_Distaverage).strip("[]"))
file.write("\n")
file.write("Interaction energy (kJ/mol)")
file.write(",")
file.write(str(Cl_Hlistaverage).strip("[]"))
file.write("\n")
file.write("Error (kJ/mol)")
file.write(",")
file.write(str(Cl_Herr).strip("[]"))
file.write("\n")
file.close()